<?php
/**
 * Country/State Lookup plugin for Joomla
 *
 * @author selfget.com (info@selfget.com)
 * @package CSLookup
 * @copyright Copyright 2009 - 2012
 * @license GNU Public License
 * @link http://www.selfget.com
 * @version 2.2
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');
jimport('joomla.html.parameter');

class plgSystemCSLookup extends JPlugin
{
  /**
  *
  * @var boolean
  * @access private
  */
  private $_cslookup = false;

  /**
  *
  * @var string
  * @access private
  */
  private $_countryid = '';

  /**
  *
  * @var string
  * @access private
  */
  private $_stateid = '';

  /**
  *
  * @var string
  * @access private
  */
  private $_statewidthstr = '';

  /**
  *
  * @var string
  * @access private
  */
  private $_statesizestr = '';

  /**
  *
  * @var string
  * @access private
  */
  private $_countrywidthstr = '';

  /**
  *
  * @var string
  * @access private
  */
  private $_countrysizestr = '';

  public function __construct(& $subject, $config)
  {
    parent::__construct($subject, $config);
    $this->loadLanguage();
  }

  public function onAfterDispatch()
  {
    // Get the application object.
    $app = JFactory::getApplication();

    // Make sure we are not in the administrator
    if ( $app->isAdmin() ) return;

    // Only site pages that are html docs
    if ( JRequest::getBool('no_html') === true ) return;

    // Check current page
    $pages = preg_split("/[\r\n]+/", $this->params->def('pagelist', ''));
    $matchfound = false;
    foreach ($pages as $pages_key => $pages_value)
    {
      $pages_value = trim($pages_value);
      if ($pages_value != '')
      {
        $parts = explode(";", $pages_value);
        $parts_params = explode(",", trim($parts[0]));
        // Check all parameters
        $allmatch = true;
        foreach ($parts_params as $par_key => $par_value)
        {
          $current_param = explode("=", trim($par_value));
          if ( JRequest::getVar(trim($current_param[0])) != trim($current_param[1]) ) { $allmatch = false; break; } // ToDo: Normalize HTML encoding chars
        }
        if ($allmatch)
        {
          // Found: register globals and exit
          $this->_cslookup = true;
          $this->_countryid = trim($parts[1]);
          $this->_stateid = trim($parts[2]);
          $matchfound = true;
        }
      }
      if ($matchfound) { break; }
    }

    if ($matchfound)
    {
      $statewidth = $this->params->def('statewidth', '');
      if ( is_numeric($statewidth) ) {
        $this->_statewidthstr = 'width="' . intval($statewidth) . '" style="width: ' . intval($statewidth) . 'px" ';
      }
      $statesize = $this->params->def('statesize', '1');
      if ( is_numeric($statesize) ) {
        $this->_statesizestr = 'size="' . intval($statesize) . '"';
      }
      $countrywidth = $this->params->def('countrywidth', '');
      if ( is_numeric($countrywidth) ) {
        $this->_countrywidthstr = 'width="' . intval($countrywidth) . '" style="width: ' . intval($countrywidth) . 'px" ';
      }
      $countrysize = $this->params->def('countrysize', '1');
      if ( is_numeric($countrysize) ) {
        $this->_countrysizestr = 'size="' . intval($countrysize) . '"';
      }
      // Add javascript to page head
      $document =& JFactory::getDocument();
      $document->addScript( JURI::root(true).'/plugins/system/cslookup/js/countries.js' );
    }

  }

  public function onAfterRender()
  {
    if ($this->_cslookup)
    {
      $content = JResponse::getBody();
      $changed = false;

      // Replace the fields
      if ($this->_countryid) {
        $onchange = '';
        $initscript = '<script type="text/javascript">populateCountriesId(\''.$this->_countryid.'\');print_country(\''.$this->_countryid.'\');' . '</script>';
        $initvalues = '';

        if ($this->_stateid != '') {
          $onchange = 'onchange="print_state(\'' . $this->_stateid . '\',this.selectedIndex);" ';
          $initscript .= '<script type="text/javascript">populateStatesId(\''.$this->_stateid.'\');print_state(\'' . $this->_stateid . '\',0);</script>';
          // Change state field from input type to select type
          $pattern = '/<input *([^>]*)(id *= *[\",\']'.preg_quote($this->_stateid).'[\",\'])([^>]*)(\/?>)/imsU';
          if (preg_match($pattern, $content, $matches)) {
            $name = preg_match('/ name *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $name_array) ? trim($name_array[0]) : '';
            $id = preg_match('/ id *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $id_array) ? trim($id_array[0]) : '';
            $class = preg_match('/ class *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $class_array) ? trim($class_array[0]) : '';
            $style = preg_match('/ style *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $style_array) ? trim($style_array[0]) : '';
            $value = preg_match('/ value *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $value_array) ? '<option value="' . trim($value_array[1]) . '" selected="selected">' . trim($value_array[1]) . '</option>' : '';
            $currentstate = $value_array[1];
            $replacement = '<select ' . $name . ' ' . $id . ' ' . $class . ' ' . $style . ' ' . $this->_statewidthstr . ' ' . $this->_statesizestr . '>' . $value . '</select>';
            $content = preg_replace($pattern, $replacement, $content);
            $changed = true;
          } else {
            // Change state field of select type
            $currentstate = '';
            $pattern = '/<select *([^>]*)(id *= *[\",\']'.preg_quote($this->_stateid).'[\",\'])([^>]*)(\/?>)(.*)<\/select>/imsU';
            if (preg_match($pattern, $content, $matches)) {
              $name = preg_match('/ name *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $name_array) ? trim($name_array[0]) : '';
              $id = preg_match('/ id *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $id_array) ? trim($id_array[0]) : '';
              $class = preg_match('/ class *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $class_array) ? trim($class_array[0]) : '';
              $style = preg_match('/ style *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $style_array) ? trim($style_array[0]) : '';
              if (preg_match('/<option *([^>]*)selected *= *[\",\']([^>]*)[\",\']([^>]*)>([^<]*)<\/option>/imsU', $matches[0], $statematches)) {
                $currentstate = preg_match('/ value *= *[\",\'](.*)[\",\'] ?/imsU', $statematches[0], $value_array) ? trim($value_array[1]) : '';
              }
              $replacement = '<select ' . $name . ' ' . $id . ' ' . $class . ' ' . $style . ' ' . $this->_statewidthstr . ' ' . $this->_statesizestr . '>';
              $content = preg_replace($pattern, $replacement, $content);
              $changed = true;
            }
          }
          $initvalues = '<script type="text/javascript">populateStateValue(\''.$this->_stateid.'\',\''.$currentstate.'\');</script>';
        }
        // Change country field from input type to select type
        $pattern = '/<input *([^>]*)(id *= *[\",\']'.preg_quote($this->_countryid).'[\",\'])([^>]*)(\/?>)/imsU';
        if (preg_match($pattern, $content, $matches)) {
          $name = preg_match('/ name *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $name_array) ? trim($name_array[0]) : '';
          $id = preg_match('/ id *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $id_array) ? trim($id_array[0]) : '';
          $class = preg_match('/ class *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $class_array) ? trim($class_array[0]) : '';
          $style = preg_match('/ style *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $style_array) ? trim($style_array[0]) : '';
          $value = preg_match('/ value *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $value_array) ? '<option value="' . trim($value_array[1]) . '" selected="selected">' . trim($value_array[1]) . '</option>' : '';
          $currentcountry = $value_array[1];
          $replacement = '<select ' . $name . ' ' . $id . ' ' . $class . ' ' . $style . ' ' . $this->_countrywidthstr . ' ' . $this->_countrysizestr . ' ' . $onchange . '>' . $value . '</select>';
          $content = preg_replace($pattern, $replacement, $content);
          $changed = true;
        } else {
          // Change state field of select type
          $currentcountry = '';
          $pattern = '/<select *([^>]*)(id *= *[\",\']'.preg_quote($this->_countryid).'[\",\'])([^>]*)(\/?>)(.*)<\/select>/imsU';
          if (preg_match($pattern, $content, $matches)) {
            $name = preg_match('/ name *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $name_array) ? trim($name_array[0]) : '';
            $id = preg_match('/ id *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $id_array) ? trim($id_array[0]) : '';
            $class = preg_match('/ class *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $class_array) ? trim($class_array[0]) : '';
            $style = preg_match('/ style *= *[\",\'](.*)[\",\'] ?/imsU', $matches[0], $style_array) ? trim($style_array[0]) : '';
            if (preg_match('/<option *([^>]*)selected *= *[\",\']([^>]*)[\",\']([^>]*)>([^<]*)<\/option>/imsU', $matches[0], $countrymatches)) {
              $currentcountry = preg_match('/ value *= *[\",\'](.*)[\",\'] ?/imsU', $countrymatches[0], $value_array) ? trim($value_array[1]) : '';
            }
            $replacement = '<select ' . $name . ' ' . $id . ' ' . $class . ' ' . $style . ' ' . $this->_countrywidthstr . ' ' . $this->_countrysizestr . ' ' . $onchange . '>';
            $content = preg_replace($pattern, $replacement, $content);
            $changed = true;
          }
        }
        
        if ($changed) {
          // Attach the init values to the head
          $initvalues = '<script type="text/javascript">populateCountryValue(\''.$this->_countryid.'\',\''.$currentcountry.'\');</script>' . $initvalues;
          $content = preg_replace('/(<\/head>)/i', $initvalues . '${1}', $content);
          // Attach the init script to the body
          $content = preg_replace('/(<\/body>)/i', $initscript . '${1}', $content);
        }

      }

      if ($changed) { JResponse::setBody($content); }
    }

  }

}
